/*
* SScript - See the sscript.doc
* (C) 1998 Drow <drow@wildstar.net>
* http://devplanet.fastethernet.net
*/

#include "sscript.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <net/if.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/file.h>
#include <signal.h>
#include <errno.h>
#include <unistd.h>
#include <netdb.h>
#include <arpa/inet.h>
#ifndef FNDELAY
#define FNDELAY O_NONBLOCK
#endif
#ifdef POSIX
#include <pwd.h>
#include <sys/utsname.h>
#endif
#ifndef SIOCGIFCONF
#include <sys/sockio.h>
#endif
#ifdef GUI
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Shell.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Dialog.h>
#include <X11/Xaw/AsciiText.h>
#include <X11/Xaw/Label.h>
#endif
#ifndef LOGFILE
#define LOGFILE "sscript.log"
#endif
#define SSCRIPT_SOCKET_FAILED             10
#define SSCRIPT_BIND_FAILED               11
#define SSCRIPT_GETSOCKETNAME_FAILED      12
#define SSCRIPT_FLAGS_FAILED              13
#define SSCRIPT_CONNECT_FAILED            20
#define SSCRIPT_UDPSEND_FAILED            31
#define SSCRIPT_UDPRECEIVE_FAILED         32
#define SSCRIPT_READ_FAILED               33
#ifdef GUI
#include <malloc.h>
#include <sys/stat.h>
#define SetArg(which, val)      XtSetArg(args[nargs], (which),\
                                         (XtArgVal) (val)); nargs++
Widget  toplevel;
Screen  *screen;
XtAppContext ac, ac2;
XEvent event, qevent;   
int displayed=0;
Display *display;

static String sscript_resources[] = {
    "*resizeGravity: NorthWest",
    "*beNiceToColormap: false",
    "*topShadowContrast: 20",
    "*bottomShadowContrast: 40",
    "*foreground: #0000FF",
    "*background: #FFFFFF",
    "*textbox*shadowWidth: 2",
    "*viewbox*shadowWidth: 2",
    "*dialog1*font: *-*-bold-i-*-*-*-*-*-*-*-*-*-*",
    "*foreground: NavyBlue",
    "*background: LightGray",
    "*viewbox*foreground: #000000",  
    "*viewbox*background: #FFFFFF",
    "*Label*foreground: #050505",
    "*textbox*foreground: #005500",
    NULL
};

static Widget msgwidget, textbox, last, entrybox, mainbox, top, dialog1, okbutton, quitbutton, continuebutton, viewbox;
static Widget form, widgets[3], choicebutton[10];

void closebox() {
    XtDestroyWidget(msgwidget);
    displayed=0;
}

int queryQuit(Widget result, XtPointer p1, XtPointer p2);
int queryOk(Widget result, XtPointer p1, XtPointer p2);
#endif

char *sscript_lindex(char *input_string, int word_number)
{
 char *tokens[255];
 static char tmpstring[512];
 int i;
 strcpy(tmpstring,input_string);
 tokens[i=0] = strtok(tmpstring, " ");
 while ((tokens[++i] = strtok(NULL, " ")));
 tokens[i] = NULL;
 return(tokens[word_number]);
}

int sscript_connect(char *server, int port, char *virtual)
{
 struct sockaddr_in address;
 struct sockaddr_in la;
 int len, sockfd;
 sockfd = socket(AF_INET, SOCK_STREAM, 0);
 if(sockfd<1) 
 {
  errno = 10;
  return -1;
 }
 address.sin_family = AF_INET;
 address.sin_addr.s_addr = inet_addr(server);
 address.sin_port = htons(port);
 len = sizeof(address);
 if(virtual!=NULL)
 {
  la.sin_family = AF_INET;
  la.sin_addr.s_addr = inet_addr(virtual);
  la.sin_port = 0;
  bind(sockfd, (struct sockaddr *)&la, sizeof(la));
 }
 if(connect(sockfd, (struct sockaddr *)&address, len)<0)
 {
  errno = 20;
  return -1;
 }
 return sockfd;
}

int sscript_server(int port)
{
 int sockfd2, listen_len;
 struct sockaddr_in listen_addr;
 sockfd2 = socket(AF_INET, SOCK_STREAM, 0);
 if(sockfd2<1)
 {
  errno = 10;
  return -1;
 }
 listen_addr.sin_family = AF_INET;
 listen_addr.sin_addr.s_addr = htonl(INADDR_ANY);
 listen_addr.sin_port = htons(port);
 listen_len = sizeof(listen_addr); 
 if(bind(sockfd2, (struct sockaddr *)&listen_addr, listen_len))
 {
  errno = 11;
  return -1;
 }
 return sockfd2;
}

int sscript_wait_clients(int sockfd2, int port, int forking)
{
 int sockfd,len,from_len,pid;
 struct sockaddr_in address;
 struct sockaddr_in from_addr;
 struct sockaddr_in listen_addr;
 listen_addr.sin_family = AF_INET;
 listen_addr.sin_addr.s_addr = htonl(INADDR_ANY);
 listen_addr.sin_port = htons(port);
 len = sizeof(address);
 listen(sockfd2, 5);
 for(;;)
 {
  if(forking)
  {
   if(sockfd!=(int)NULL) close(sockfd);
  }
  sockfd = accept(sockfd2, (struct sockaddr *)&address, &len);
  if(forking)
  {
   if(pid=fork()) break;
  }
 }
 from_len=sizeof(from_addr);
 memset(&from_addr, 0, sizeof(from_addr));
 if(getpeername(sockfd, (struct sockaddr *)&from_addr,&from_len) < 0)
 {
  strcpy(remoteIP,"unknown");
 }
 else
 {
  strcpy(remoteIP,inet_ntoa(from_addr.sin_addr));
 }
 return sockfd;
}

char *sscript_get_remote_ip()
{
 return remoteIP;
}

int sscript_disconnect(int sockfd)
{
 shutdown(sockfd,2);
 close(sockfd);
 return 0;
}

int sscript_dump(int sockfd, char *filename)
{
 char temp[1024]="";
 FILE *fpa;
 fpa=fopen(filename,"r");
 while(fgets(temp,1024,fpa)!=NULL)
  write(sockfd, temp, strlen(temp));
 if(fpa!=NULL) fclose(fpa);
 return 0;
}

int sscript_ping(char *hostname)
{
 struct sockaddr_in other_addr;
 int sockfd, result;
 char temp[255];
 sockfd=socket(AF_INET, SOCK_STREAM, 0);
 other_addr.sin_family = AF_INET;
 other_addr.sin_addr.s_addr = inet_addr(hostname);
 other_addr.sin_port = htons(7);
 connect(sockfd, (struct sockaddr*) &other_addr,sizeof(other_addr));
 result=write(sockfd,"ping\n",strlen("ping\n"));
 result=read(sockfd,temp,result);
 if(sockfd!=(int)NULL) close(sockfd);
 return 0;
}

int sscript_test(char *hostname, int port)
{
 int sockfd;
 struct sockaddr_in other_addr;
 if((sockfd=socket(AF_INET, SOCK_STREAM, 0))<0)
 {
  errno = 10;
  return -1;
 }
 other_addr.sin_family = AF_INET;
 other_addr.sin_addr.s_addr = inet_addr(hostname);
 other_addr.sin_port = htons(port);
 if(connect(sockfd, (struct sockaddr*)&other_addr,sizeof(other_addr))==-1)
 {
  errno = 20;
  return -1;
 }
 else return 0;
 if(sockfd!=(int)NULL) close(sockfd);
}

char *sscript_version()
{
 return "Socket Script library version 1.4 by Drow <drow@wildstar.net>";
}

char *sscript_read(int sockfd, int chop)
{
 int i, result;
 char inchar;
 char string[1024];
 bzero(string,1024);
 strcpy(string,"");
 for(i=0;(result=read(sockfd,&inchar,1))!='\0';i++)
 {
  string[i]=inchar;
  if(inchar=='\n') break;
 }
 if (chop) string[i-1]=' ';
 return (char *)string;
}

int sscript_wait(int seconds) 
{
 sleep(seconds);
 return 0;
}

void sscript_write(int sockfd, char *string)
{
 write(sockfd, string, strlen(string));
}

int sscript_compare(char *case1, char *case2)
{
 return (strcasecmp(case1,case2));
}

char *sscript_lrange(char *input_string, int starting_at)
{
 char global_var[512];
 char *tokens[555];
 static char tmpstring[512]="";
 int i;
 char out_string[512]="";
 strcpy(out_string,"");
 if(input_string==NULL) {
  strcpy(out_string," ");
  strcat(out_string,NULL);
  strcpy(global_var,out_string);
  return (char *)global_var; }
 strcpy(tmpstring,input_string);
 tokens[i=0] = strtok(tmpstring, " ");
 while((tokens[++i] = strtok(NULL, " ")));
 tokens[i] = NULL;
 i++;
 if(i<starting_at) return (int)NULL;
 while(tokens[starting_at] != NULL)
 {
  strcat(out_string,tokens[starting_at]);
  strcat(out_string, " ");
  starting_at++;
 }
 strcpy(global_var,out_string);
 return (char *)global_var;
}

int sscript_match(char *masked, char *orig)
{
 while(*masked == '*' || tolower(*masked)==tolower(*orig) || *masked == '?') 
  if(*masked == '*')
   if(*++masked) {
    while(*orig)
     if(!sscript_match(masked,orig++)) return 0;
      return 1;
   }
   else
    return 0;
   else if (!*masked)
    return 0;
   else if (!*orig)
    return 1;
   else {
    ++masked;
    ++orig;
   }
 return 1;
}

void sscript_appendlog(char *string)
{
 FILE *fdlog;
 fdlog=fopen(LOGFILE,"a");
 fputs(string,fdlog);
 if(fdlog!=NULL) fclose(fdlog);
}

int sscript_udp_send(char *hostname, int port, char *msg)
{
 int udpsock,i;
 struct sockaddr_in udpaddr;
 udpsock = socket(AF_INET, SOCK_DGRAM, 0);
 if(udpsock<0)
 {
  errno = 10;
  return -1;
 }
 udpaddr.sin_family = AF_INET;
 udpaddr.sin_port = htons(port);
 udpaddr.sin_addr.s_addr = inet_addr(hostname);
 if(sendto(udpsock,msg,sizeof(msg),0,(struct sockaddr *)&udpaddr,sizeof(udpaddr))<0)
 {
  errno = 30;
  return -1;
 }
 return 0;
}

char *sscript_udp_listen(int port)
{
 int udpsock,len;
 struct sockaddr_in udpaddr, from;
 char msg[255];
 udpsock = socket(AF_INET, SOCK_DGRAM, 0);
 if(udpsock<0)
 {
  errno = 10;
  return (char *)NULL;
 }
 udpaddr.sin_family = AF_INET;
 udpaddr.sin_addr.s_addr = INADDR_ANY;
 udpaddr.sin_port = htons(port);
 if(bind(udpsock,(struct sockaddr *)&udpaddr,sizeof(udpaddr))<0)
 {
  errno = 11;
  return (char *)NULL;
 }
 len = sizeof(from);
 if(recvfrom(udpsock,msg,sizeof(msg),0,(struct sockaddr *)&from,&len)<0)
 {
  errno = 32;
  return (char *)NULL;
 }
 return (char *)msg;
 close(udpsock);
}

char *sscript_icmp_detect()
{
 int icmpsock,len,result,type;
 struct sockaddr_in icmpaddr;
 char readbuf[1024]="";
 char msg[255];
 if((icmpsock=socket(AF_INET, SOCK_RAW, IPPROTO_ICMP))<0)
 {
  errno = 10;
  return (char *)NULL;
 }
 icmpaddr.sin_family = AF_INET;
 icmpaddr.sin_addr.s_addr = INADDR_ANY;
 icmpaddr.sin_port = 0;
 if(bind(icmpsock,(struct sockaddr *)&icmpaddr,sizeof(icmpaddr))<0)
 {
  errno = 11;
  return (char *)NULL;
 }
 len=sizeof(icmpaddr);
 if(getsockname(icmpsock,(struct sockaddr *)&icmpaddr,&len)<0)
 {
  errno = 12;
  return (char *)NULL;
 }
 if((result=read(icmpsock,readbuf,sizeof(readbuf)))<0)
 {
  errno = 33;
  return (char *)NULL;
 }
 type=readbuf[20] & 0xff;
 sprintf(msg,"%d %d.%d.%d.%d ",type,readbuf[12]&0xff,readbuf[13]&0xff,readbuf[14]&0xff,readbuf[15]&0xff);
 return (char *)msg;
 close(icmpsock);
}

char *sscript_resolve_host(char *hostname)
{
 struct hostent *hp;
 struct sockaddr_in from;
 char result[255];
 unsigned long dest;
 memset(&from, 0, sizeof(struct sockaddr_in));
 from.sin_family = AF_INET;
 hp=gethostbyname(hostname);
 if(hp==NULL) strcpy(result,"unknown");
 else
 {
  memcpy(&from.sin_addr,hp->h_addr,hp->h_length);
  strcpy(result,inet_ntoa(from.sin_addr));
 }
 return (char *)result;
}

char *sscript_resolve_ip(char *ip)
{
 struct hostent *hp;
 struct sockaddr_in from;
 char result[255];
 unsigned long dest;
 from.sin_family = AF_INET;
 from.sin_addr.s_addr = inet_addr(ip);
 hp=gethostbyaddr((char *)&from.sin_addr, sizeof(struct in_addr),from.sin_family);
 if(hp==NULL) strcpy(result,"unknown");
 else strcpy(result,(char *)hp->h_name);;
 return (char *)result;
}

char *sscript_get_localhost()
{
 char result[255];
 gethostname(result,sizeof(result));
 return (char *)result;
}

#ifdef GUI
int queryOk(Widget result, XtPointer p1, XtPointer p2)
{
 strcpy(queryValue,XawDialogGetValueString(dialog1));
 XtDestroyWidget(mainbox);
 XtUnrealizeWidget(top);
 XtDestroyApplicationContext(ac);
 displayed = 0;
}

int ContinueIt(Widget result, XtPointer p1, XtPointer p2)
{
 XtDestroyWidget(mainbox);
 XtUnrealizeWidget(top);
 XtDestroyApplicationContext(ac);
 displayed = 0;
}

int queryQuit(Widget result, XtPointer p1, XtPointer p2)
{
 XtDestroyWidget(mainbox);
 exit(0);
}

int sscript_query(char *string, int argc1, char *argv1[])
{
 if(displayed) return 0;
 displayed = 1;
 top = XtVaAppInitialize(&ac, "SScript", NULL, 0, &argc1, argv1, sscript_resources, NULL);
 mainbox = XtVaCreateManagedWidget("mainbox", boxWidgetClass, top, NULL);
 dialog1 = XtVaCreateManagedWidget("dialog1", dialogWidgetClass, mainbox, XtNlabel, string, XtNvalue, "", NULL);
 okbutton = XtVaCreateManagedWidget("OK", commandWidgetClass, dialog1, NULL); 
 quitbutton = XtVaCreateManagedWidget("Quit", commandWidgetClass, dialog1, NULL);
 XtAddCallback(okbutton, XtNcallback, (XtPointer)queryOk, NULL);
 XtAddCallback(quitbutton, XtNcallback, (XtPointer)queryQuit, NULL);
 XtRealizeWidget(top);
 while(displayed) {
  XtAppNextEvent(ac, &qevent);
  XtDispatchEvent(&qevent);
 }
}

int sscript_viewfile(char *file, int argc1, char *argv1[])
{
 FILE *fd;
 char *file_buf;
 off_t len;
 struct stat stat_buf;
 if(displayed) return 0; 
 displayed = 1;
 top = XtVaAppInitialize(&ac, "SScript", NULL, 0, &argc1, argv1, sscript_resources, NULL);
 mainbox = XtVaCreateManagedWidget("mainbox", boxWidgetClass, top, NULL);
 continuebutton = XtVaCreateManagedWidget("Continue",commandWidgetClass,mainbox,NULL);
 viewbox = XtVaCreateManagedWidget("viewbox", asciiTextWidgetClass, mainbox, XtNheight, 300, XtNwidth, 500, XtNscrollVertical, XawtextScrollWhenNeeded, XtNscrollHorizontal, XawtextScrollWhenNeeded, NULL);
 XtAddCallback(continuebutton, XtNcallback, (XtPointer)ContinueIt, NULL);
 if((fd=fopen(file,"ro"))==NULL)
  XtVaSetValues(viewbox, XtNstring, "Can't open that file", NULL);
 else
 {
  fstat(fileno(fd), &stat_buf);
  len = stat_buf.st_size;
  file_buf = malloc(len * sizeof(char) +1);
  fread(file_buf, 1, len, fd);
  file_buf[len] = '\0';
  XtVaSetValues(viewbox, XtNstring, file_buf, NULL);
  free(file_buf);
  fclose(fd);
 }
 XtRealizeWidget(top);
 while(displayed) {
  XtAppNextEvent(ac, &qevent);
  XtDispatchEvent(&qevent);
 }
}

int sscript_msgbox(char *title, char *label, char *textstring, int len)
{
    Arg args[20];
    register int nargs, nwidgets;
    int width;
    XtCallbackRec button1[2]; 
 
    if (displayed)
        return;
    XFlush(display);
    displayed=1;

    bzero(button1, sizeof(button1));
    
    button1[0].callback = closebox;
    button1[0].closure = (caddr_t) 0;
    
    nargs = 0;
    SetArg(XtNx, 10);
    SetArg(XtNy, 10);        
    SetArg(XtNtitle, title);
    SetArg(XtNmaxHeight, 130);
    SetArg(XtNminHeight, 120);
    SetArg(XtNmaxWidth, (len+25)*8);
    SetArg(XtNminWidth, (len+20)*8);
    
    msgwidget=XtCreateWidget("window",
                             applicationShellWidgetClass,
                             toplevel, args, nargs);
    nargs = 0;
    form = XtCreateWidget("box", boxWidgetClass,
                          msgwidget, args, nargs);
    XtManageChild(form);
    
    nargs = 0;
    SetArg(XtNlabel,label);
    SetArg("borderWidth", 0);
    SetArg(XtNjustify,XtJustifyCenter);
    SetArg(XtNwidth,(len+20)*8);
    SetArg(XtNheight,15);
    XtManageChild(last=XtCreateWidget("label", labelWidgetClass,
                          form, args, nargs));
    
    nargs = 0;
    
    SetArg(XtNlabel, textstring);
    SetArg(XtNborderWidth, 1);
    SetArg(XtNjustify,XtJustifyCenter);
    SetArg(XtNfromVert,last);
    SetArg(XtNwidth,(len+20)*8);
    SetArg(XtNheight,60);
    
    XtManageChild(textbox=XtCreateWidget("textbox", labelWidgetClass,
                          form, args, nargs));
    
    nwidgets = 0;
    
    nargs = 0;
    SetArg(XtNcallback, button1);
    SetArg(XtNfromVert, textbox);
    SetArg(XtNhighlightThickness, -1);
    SetArg(XtNwidth, (len+20)*8);
    widgets[nwidgets++] = XtCreateWidget("OK",
                                         commandWidgetClass,
                                         form, args, nargs);

    XtManageChild(form);
    XtManageChildren(widgets, nwidgets);
    XtRealizeWidget(msgwidget);
    
    XtPopup(msgwidget, XtGrabExclusive);
    
    while(displayed) {
        XtAppNextEvent(ac2, &event);
        XtDispatchEvent(&event); 
    }
    XFlush(display);
}

int sscript_gui_init(int argc, char *argv[])
{
 toplevel = XtAppInitialize(&ac2, "XMSG", NULL, 0, &argc, argv, sscript_resources, NULL, 0);   
 display = XtDisplay(toplevel);
 screen  = XtScreen(toplevel);
}

int choiceHdl(Widget result, XtPointer p1, XtPointer p2)
{
 if(result==choicebutton[0]) strcpy(queryValue,"1");
 else if(result==choicebutton[1]) strcpy(queryValue,"2");
 else if(result==choicebutton[2]) strcpy(queryValue,"3");
 else if(result==choicebutton[3]) strcpy(queryValue,"4");
 else if(result==choicebutton[4]) strcpy(queryValue,"5");
 else if(result==choicebutton[5]) strcpy(queryValue,"6");
 else if(result==choicebutton[6]) strcpy(queryValue,"7");
 else if(result==choicebutton[7]) strcpy(queryValue,"8");
 else if(result==choicebutton[8]) strcpy(queryValue,"9");
 else if(result==choicebutton[9]) strcpy(queryValue,"10");
 else strcpy(queryValue,"-1");
 XtDestroyWidget(mainbox);
 XtUnrealizeWidget(top);
 XtDestroyApplicationContext(ac);
 displayed = 0;
}  

int sscript_choicebox(int argc1, char *argv1[], char *temp)
{
 int i;
 if(displayed) return 0;
 displayed = 1;
 top = XtVaAppInitialize(&ac, "SScript", NULL, 0, &argc1, argv1, sscript_resources, NULL);
 mainbox = XtVaCreateManagedWidget("mainbox", boxWidgetClass, top, NULL);
 for(i=0;sscript_lindex(temp,i)!=NULL;i++)
 {
  if(!sscript_compare(sscript_lindex(temp,i),"|"))
  {
   last = XtVaCreateManagedWidget("label", labelWidgetClass, mainbox, XtNlabel, sscript_lrange(temp,i+1), "borderWidth", 0, NULL);
   break;
  }
 }   
 i=0;
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 if(sscript_lindex(temp,i)!=NULL && *sscript_lindex(temp,i)!='|')
 {
 choicebutton[i] = XtVaCreateManagedWidget(sscript_lindex(temp,i), commandWidgetClass, mainbox, NULL);
 XtAddCallback(choicebutton[i], XtNcallback, (XtPointer)choiceHdl, NULL);
 i++;
 }
 XtRealizeWidget(top);
 while(displayed) {
  XtAppNextEvent(ac, &qevent);
  XtDispatchEvent(&qevent);
 }
}
#endif

char *sscript_devstat(char *device)
{/* based on Trevor F. Linton's code */
 int sockfd=0, device_flags=0, found=0;
 char lbuf[255], *blen, *blenmax, result[255];   
 struct ifreq ifreq, *ifr;
 struct ifconf ifc;
 if((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
 {
  sprintf(result, "SOCKET FAILED: %s", strerror(errno));
  return (char *)result;
 }
 ifc.ifc_len = sizeof(lbuf);
 ifc.ifc_buf = lbuf;
 if(ioctl(sockfd, SIOCGIFCONF, (char *)&ifc) < 0)
 {
  sprintf(result, "IOCTL FAILED: %s", strerror(errno));
  return (char *)result;
 }
 ifr = ifc.ifc_req;
 blenmax = lbuf + ifc.ifc_len;
 for(blen = lbuf; blen < blenmax; blen = blen + sizeof(ifr->ifr_name) + sizeof(ifr->ifr_addr))
 {
  ifr = (struct ifreq *)blen;
  if(strcmp(ifr->ifr_name, device)) continue;
  found = 1;
  ifreq = *ifr;
  if(ioctl(sockfd, SIOCGIFFLAGS, (char *)&ifreq) < 0)
  {
   sprintf(result, "IOCTL FAILED: %s", strerror(errno));
   return (char *)result;
  }
  device_flags = ifreq.ifr_flags;
  sprintf(result, "%s ", ifreq.ifr_name);
  if((device_flags & IFF_PROMISC) != 0) strcat(result, "1 ");
  else strcat(result, "0 ");
  if((device_flags & IFF_UP) != 0) strcat(result, "1 ");
  else strcat(result, "0 ");
  if((device_flags & IFF_POINTOPOINT) != 0) strcat(result, "1 ");
  else strcat(result, "0 ");
  if((device_flags & IFF_BROADCAST) != 0) strcat(result, "1 ");
  else strcat(result, "0 ");
  if((device_flags & IFF_MULTICAST) != 0) strcat(result, "1 ");
  else strcat(result, "0 ");
  if((device_flags & IFF_ALLMULTI) != 0) strcat(result, "1 ");
  else strcat(result, "0 ");
  if((device_flags & IFF_NOARP) != 0) strcat(result, "1");   
  else strcat(result, "0");
 }
 if(found==0) sprintf(result, "%s: Unknown device.", device);
 return (char *)result;
}

int sscript_binary_send(int sockfd, char *string)
{
 char temp4[255], temp2[255];
 int cnt;
 FILE *fpa;
 sprintf(temp4,"uuencode %s %s > %s/.temp.uue 2>/dev/null",string,string,TMP_DIR);
 system(temp4);
 sprintf(temp2,"%s/.temp.uue",TMP_DIR);
 fpa=fopen(temp2,"r");
 if(fpa==NULL || fileno(fpa)<0) return -1;
 else {
  while((cnt = read(fileno(fpa), temp4, 250))>0)
  write(sockfd, temp4, cnt);
  if(fpa!=NULL) fclose(fpa);
  sprintf(temp4,"rm -f %s/.temp.uue",TMP_DIR);
  system(temp4);
 }
}

int sscript_binary_get(int sockfd)
{
 char temp2[255], temp4[255], inchar, inall[1024];
 FILE *fpa;
 int i;
 sprintf(temp2,"%s/.temp.uue",TMP_DIR);
 fpa=fopen(temp2,"w");
 if(fpa==NULL || fileno(fpa)<0) return -1;
 while(strcasecmp(inall,"end\n"))
 {
  bzero(inall, 1024);
  for(i=0;read(sockfd,&inchar,1)!='\0';i++)
  {
   inall[i]=inchar;
   if(inchar=='\n') break;
  }
  fputs(inall,fpa);
 }
 if(fpa!=NULL) fclose(fpa);
 sprintf(temp4,"uudecode %s/.temp.uue",TMP_DIR);
 system(temp4);
 sprintf(temp4,"rm -f %s/.temp.uue",TMP_DIR);
 system(temp4);
}

char *sscript_login_to_passwd(char *login)
{
#ifdef POSIX
 struct passwd *pw;
 pw = getpwnam(login);
 if(pw!=NULL) return (char *)pw->pw_passwd;
#endif
}

char *sscript_uid_to_login(long my_uid)
{ 
#ifdef POSIX
 struct passwd *pw;
 pw = getpwuid(my_uid);
 if(pw!=NULL) return (char *)pw->pw_name;
#endif
}

int sscript_sokstat(char *option, int sockfd)
{  
 char data[255]; 
 int optlen=sizeof(int),optval=1;
 if(!strcasecmp(option,"sendbuf")) getsockopt(sockfd, SOL_SOCKET, SO_SNDBUF, (char *)&optval, &optlen);
 else if(!strcasecmp(option,"recvbuf")) getsockopt(sockfd, SOL_SOCKET, SO_RCVBUF, (char *)&optval, &optlen);
 else if(!strcasecmp(option,"error")) getsockopt(sockfd, SOL_SOCKET, SO_ERROR, (char *)&optval, &optlen);
 else if(!strcasecmp(option,"type")) getsockopt(sockfd, SOL_SOCKET, SO_TYPE, (char *)&optval, &optlen);
 else optval=-1;
 return optval;
}

char *sscript_time_read(int sockfd, int time_sec)
{
 struct timeval timeout;
 int max_fd;
 fd_set readfs, newfs;
 timeout.tv_sec=time_sec;
 timeout.tv_usec=0;
 FD_ZERO(&readfs);
 FD_SET(sockfd, &readfs);
 max_fd = sockfd;
 memcpy(&newfs, &readfs, sizeof(readfs));
 select(max_fd+1, &newfs, NULL, NULL, &timeout);
 if(FD_ISSET (sockfd, &newfs))
 {
  read(sockfd, global_var, sizeof(global_var));
  return(global_var); 
 }
 return("timeout");
}

int sscript_redir(int sockfd, int rsck)
{
 char buf[4096];
 fd_set readfs, newfs;
 int max_fd, len;
 FD_ZERO(&readfs);
 FD_SET(sockfd, &readfs);
 FD_SET(rsck, &readfs);
 if(sockfd>rsck) max_fd = sockfd;
 else max_fd = rsck;
 while(1) {
  memcpy(&newfs, &readfs, sizeof(readfs));
  select(max_fd+1, &newfs, NULL, NULL, NULL);
  if(FD_ISSET(sockfd, &newfs))
  {
   if((len=read(sockfd, buf, sizeof(buf)))<1) break;
   if(write(rsck, buf, len)!=len) break;
  }
  if(FD_ISSET(rsck, &newfs))
  {
   if((len=read(rsck, buf, sizeof(buf)))<1) break;
   if(write(sockfd, buf, len)!=len) break;
  }
 }
}

int sscript_nodelay(int sockfd)
{
 int i;
 if(( i = fcntl(sockfd, F_GETFL, 0)) == -1);
 else if (fcntl(sockfd, F_SETFL, i | FNDELAY) == -1);
}

